/* Nokia 5110 LCD AVR Library example
 *
 * Copyright (C) 2015 Sergey Denisov.
 * Written by Sergey Denisov aka LittleBuster (DenisovS21@gmail.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public Licence
 * as published by the Free Software Foundation; either version 3
 * of the Licence, or (at your option) any later version.
 *
 * Original library written by SkewPL, http://skew.tk
 */

#define F_CPU     800000UL
#define BAUD      14400
#define BRC       ((F_CPU/8/BAUD) - 1)

#include <stdio.h>
#include <stdlib.h>
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#include <util/delay.h>
#include <util/setbaud.h> 
#include <avr/io.h>
#include <avr/interrupt.h>

#include "../nokia5110.h"
#include "../dht22.h"
#include "../light_ws2812.h"



int main(void) {

	uint8_t state = -1;
	uint16_t photoresistorValue;
	char buffer1[32];
	char buffer2[32];
	char buffer3[32];
	struct dht22 reader;
	float temp, hum;

	struct cRGB led_array_on[] = {
		{255, 255, 255},
		{255, 0, 0},
		{0, 255, 0},
		{0, 0, 255},
		{0, 0, 255},
		{0, 255, 0},
		{255, 0, 0},
		{255, 255, 255}
	};

	struct cRGB led_array_off[] = {
		{0, 0, 0},
		{0, 0, 0},
		{0, 0, 0},
		{0, 0, 0},
		{0, 0, 0},
		{0, 0, 0},
		{0, 0, 0},
		{0, 0, 0}
	};

	DDRD |= (1 << PD0); //set DDR0 as output
	DDRD |= (1 << PD1);

	ADMUX = 0;
	ADMUX |= (1 << MUX0);          // ADC1 - channel 1
	ADMUX |= (1 << REFS0);         // AVCC with external capacitor at AREF pin
	ADCSRA = 0;
	ADCSRA |= (1 << ADPS2) | (1 << ADPS2) | (1 << ADPS0);
	ADCSRA |= (1 << ADEN);         // enable ADC
	
	dht_init(&reader, PB6);
	nokia_lcd_init();

	while (1) {
		ADCSRA |= (1 << ADSC);         //start conversion
		loop_until_bit_is_set(ADCSRA, ADIF);  //wait until conversion is done: until ADIF from ADCSRA is set to 1
		photoresistorValue = ADC; //read ADC value in
		sprintf(buffer1, "Lux: %d", photoresistorValue / 5);

		dht_read_data(&reader, &temp, &hum);
		sprintf(buffer2, "Temp: %.3f", temp);
		sprintf(buffer3, "Hum: %.3f", hum);

		if (photoresistorValue < 400)
			ws2812_setleds_pin(&led_array_on, 8, _BV(PB0));
		else
			ws2812_setleds_pin(&led_array_off, 8, _BV(PB0));

		if (temp < (float) 7 || temp >= (float) 30)
			PORTD &= ~(1 << PD0);
		else
			PORTD |= (1 << PD0);

		if (temp > (float) 25) {
			state = -1;
			PORTD &= ~(1 << PD1);
		} else if (temp <= (float) 25 && state == -1) {
			state = 0;
			PORTD |= (1 << PD1);
		} else if (temp <= (float) 25 && state <= 30 && state != -1) {
			state++;
		} else if (state == 30) {
			state = 0;
		}

		nokia_lcd_clear();
    	nokia_lcd_write_string("IT'S WORKING!", 1);
    	nokia_lcd_set_cursor(0, 10);
    	nokia_lcd_write_string(buffer1, 1);
    	nokia_lcd_set_cursor(0, 20);
    	nokia_lcd_write_string(buffer2, 1);
    	nokia_lcd_set_cursor(0, 30);
    	nokia_lcd_write_string(buffer3, 1);
    	nokia_lcd_render();




    	_delay_ms(100);
	}
}

